Class: Generators::DeclareSchema::Migration::Migrator

Inherits:
Object
  • Object
show all
Defined in:
lib/generators/declare_schema/migration/migrator.rb

Defined Under Namespace

Classes: Error

Class Attribute Summary collapse

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(ambiguity_resolver = {}) ⇒ Migrator

Returns a new instance of Migrator.



115
116
117
118
119
# File 'lib/generators/declare_schema/migration/migrator.rb', line 115

def initialize(ambiguity_resolver = {})
  @ambiguity_resolver = ambiguity_resolver
  @drops = []
  @renames = nil
end

Class Attribute Details

.active_record_class ⇒ Object

Returns the value of attribute active_record_class.



77
78
79
# File 'lib/generators/declare_schema/migration/migrator.rb', line 77

def active_record_class
  @active_record_class
end

.disable_constraints ⇒ Object

Returns the value of attribute disable_constraints.



77
78
79
# File 'lib/generators/declare_schema/migration/migrator.rb', line 77

def disable_constraints
  @disable_constraints
end

.disable_indexing ⇒ Object

Returns the value of attribute disable_indexing.



77
78
79
# File 'lib/generators/declare_schema/migration/migrator.rb', line 77

def disable_indexing
  @disable_indexing
end

.ignore_models ⇒ Object

Returns the value of attribute ignore_models.



77
78
79
# File 'lib/generators/declare_schema/migration/migrator.rb', line 77

def ignore_models
  @ignore_models
end

.ignore_tables ⇒ Object

Returns the value of attribute ignore_tables.



77
78
79
# File 'lib/generators/declare_schema/migration/migrator.rb', line 77

def ignore_tables
  @ignore_tables
end

Instance Attribute Details

#renames ⇒ Object

Returns the value of attribute renames.



121
122
123
# File 'lib/generators/declare_schema/migration/migrator.rb', line 121

def renames
  @renames
end

Class Method Details

.always_ignore_tables ⇒ Object



226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
# File 'lib/generators/declare_schema/migration/migrator.rb', line 226

def self.always_ignore_tables
  sessions_table =
    begin
      if defined?(CGI::Session::ActiveRecordStore::Session) &&
         defined?(ActionController::Base) &&
         ActionController::Base.session_store == CGI::Session::ActiveRecordStore
        CGI::Session::ActiveRecordStore::Session.table_name
      end
    rescue
      nil
    end

  [
    'schema_info',
    ActiveRecord::Base.try(:schema_migrations_table_name) || 'schema_migrations',
    ActiveRecord::Base.try(:internal_metadata_table_name) || 'ar_internal_metadata',
    sessions_table
  ].compact
end

.connection ⇒ Object



96
97
98
# File 'lib/generators/declare_schema/migration/migrator.rb', line 96

def connection
  ActiveRecord::Base.connection
end

.default_migration_name ⇒ Object



90
91
92
93
94
# File 'lib/generators/declare_schema/migration/migrator.rb', line 90

def default_migration_name
  existing = Dir["#{Rails.root}/db/migrate/*declare_schema_migration*"]
  max = existing.grep(/([0-9]+)\.rb$/) { Regexp.last_match(1).to_i }.max.to_i
  "declare_schema_migration_#{max + 1}"
end

.fix_native_types(types) ⇒ Object



100
101
102
103
104
105
106
107
108
# File 'lib/generators/declare_schema/migration/migrator.rb', line 100

def fix_native_types(types)
  case connection.class.name
  when /mysql/i
    types[:integer][:limit] ||= 11
    types[:text][:limit]    ||= 0xffff
    types[:binary][:limit]  ||= 0xffff
  end
  types
end

.native_types ⇒ Object



110
111
112
# File 'lib/generators/declare_schema/migration/migrator.rb', line 110

def native_types
  @native_types ||= fix_native_types(connection.native_database_types)
end

.run(renames = {}) ⇒ Object



84
85
86
87
88
# File 'lib/generators/declare_schema/migration/migrator.rb', line 84

def run(renames = {})
  g = Migrator.new
  g.renames = renames
  g.generate
end

Instance Method Details

#change_column_back(table, col_name) ⇒ Object



575
576
577
578
# File 'lib/generators/declare_schema/migration/migrator.rb', line 575

def change_column_back(table, col_name)
  type, options = column_options_from_reverted_table(table, col_name)
  "change_column :#{table}, :#{col_name}, #{type}#{', ' + options.strip if options}"
end

#change_foreign_key_constraints(model, old_table_name) ⇒ Object



482
483
484
485
486
487
488
489
490
491
492
493
494
495
496
497
498
499
500
501
502
503
504
505
506
# File 'lib/generators/declare_schema/migration/migrator.rb', line 482

def change_foreign_key_constraints(model, old_table_name)
  ActiveRecord::Base.connection.class.name.match?(/SQLite3Adapter/) and raise 'SQLite does not support foreign keys'
  return [[], []] if Migrator.disable_indexing

  new_table_name = model.table_name
  existing_fks = ::DeclareSchema::Model::ForeignKeySpec.for_model(model, old_table_name)
  model_fks = model.constraint_specs
  add_fks = model_fks - existing_fks
  drop_fks = existing_fks - model_fks
  undo_add_fks = []
  undo_drop_fks = []

  add_fks.map! do |fk|
    # next if fk.parent.constantize.abstract_class || fk.parent == fk.model.class_name
    undo_add_fks << remove_foreign_key(old_table_name, fk.options[:constraint_name])
    fk.to_add_statement
  end.compact

  drop_fks.map! do |fk|
    undo_drop_fks << fk.to_add_statement
    remove_foreign_key(new_table_name, fk.options[:constraint_name])
  end

  [drop_fks + add_fks, undo_add_fks + undo_drop_fks]
end

#change_indexes(model, old_table_name) ⇒ Object



443
444
445
446
447
448
449
450
451
452
453
454
455
456
457
458
459
460
461
462
463
464
465
466
467
468
469
470
471
472
473
474
# File 'lib/generators/declare_schema/migration/migrator.rb', line 443

def change_indexes(model, old_table_name)
  return [[], []] if Migrator.disable_constraints

  new_table_name = model.table_name
  existing_indexes = ::DeclareSchema::Model::IndexSpec.for_model(model, old_table_name)
  model_indexes_with_equivalents = model.index_specs_with_primary_key
  model_indexes = model_indexes_with_equivalents.map do |i|
    if i.explicit_name.nil?
      if ex = existing_indexes.find { |e| i != e && e.equivalent?(i) }
        i.with_name(ex.name)
      end
    end || i
  end
  existing_has_primary_key = existing_indexes.any? { |i| i.name == 'PRIMARY_KEY' }
  model_has_primary_key    = model_indexes.any?    { |i| i.name == 'PRIMARY_KEY' }

  add_indexes_init = model_indexes - existing_indexes
  drop_indexes_init = existing_indexes - model_indexes
  undo_add_indexes = []
  undo_drop_indexes = []
  add_indexes = add_indexes_init.map do |i|
    undo_add_indexes << drop_index(old_table_name, i.name) unless i.name == "PRIMARY_KEY"
    i.to_add_statement(new_table_name, existing_has_primary_key)
  end
  drop_indexes = drop_indexes_init.map do |i|
    undo_drop_indexes << i.to_add_statement(old_table_name, model_has_primary_key)
    drop_index(new_table_name, i.name) unless i.name == "PRIMARY_KEY"
  end.compact

  # the order is important here - adding a :unique, for instance needs to remove then add
  [drop_indexes + add_indexes, undo_add_indexes + undo_drop_indexes]
end

#change_table(model, current_table_name) ⇒ Object



352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
370
371
372
373
374
375
376
377
378
379
380
381
382
383
384
385
386
387
388
389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
405
406
407
408
409
410
411
412
413
414
415
416
417
418
419
420
421
422
423
424
425
426
427
428
429
430
431
432
433
434
435
436
437
438
439
440
441
# File 'lib/generators/declare_schema/migration/migrator.rb', line 352

def change_table(model, current_table_name)
  new_table_name = model.table_name

  db_columns = model.connection.columns(current_table_name).index_by(&:name)
  key_missing = db_columns[model.primary_key].nil? && model.primary_key.present?
  if model.primary_key.present?
    db_columns.delete(model.primary_key)
  end

  model_column_names = model.field_specs.keys.map(&:to_s)
  db_column_names = db_columns.keys.map(&:to_s)

  to_add = model_column_names - db_column_names
  to_add += [model.primary_key] if key_missing && model.primary_key.present?
  to_remove = db_column_names - model_column_names
  to_remove -= [model.primary_key.to_sym] if model.primary_key.present?

  to_rename = extract_column_renames!(to_add, to_remove, new_table_name)

  db_column_names -= to_rename.keys
  db_column_names |= to_rename.values
  to_change = db_column_names & model_column_names

  renames = to_rename.map do |old_name, new_name|
    "rename_column :#{new_table_name}, :#{old_name}, :#{new_name}"
  end
  undo_renames = to_rename.map do |old_name, new_name|
    "rename_column :#{new_table_name}, :#{new_name}, :#{old_name}"
  end

  to_add = to_add.sort_by { |c| model.field_specs[c]&.position || 0 }
  adds = to_add.map do |c|
    args =
      if (spec = model.field_specs[c])
        options = fk_field_options(model, c).merge(spec.sql_options)
        [":#{spec.sql_type}", *format_options(options, spec.sql_type)]
      else
        [":integer"]
      end
    "add_column :#{new_table_name}, :#{c}, #{args.join(', ')}"
  end
  undo_adds = to_add.map do |c|
    "remove_column :#{new_table_name}, :#{c}"
  end

  removes = to_remove.map do |c|
    "remove_column :#{new_table_name}, :#{c}"
  end
  undo_removes = to_remove.map do |c|
    revert_column(current_table_name, c)
  end

  old_names = to_rename.invert
  changes = []
  undo_changes = []
  to_change.each do |c|
    col_name = old_names[c] || c
    col = db_columns[col_name]
    spec = model.field_specs[c]
    if spec.different_to?(col) # TODO: DRY this up to a diff function that returns the differences. It's different if it has differences. -Colin
      change_spec = fk_field_options(model, c)
      change_spec[:limit]     ||= spec.limit   if (spec.sql_type != :text ||
                                                 ::DeclareSchema::Model::FieldSpec.mysql_text_limits?) &&
                                                  (spec.limit || col.limit)
      change_spec[:precision] = spec.precision unless spec.precision.nil?
      change_spec[:scale]     = spec.scale     unless spec.scale.nil?
      change_spec[:null]      = spec.null      unless spec.null && col.null
      change_spec[:default]   = spec.default   unless spec.default.nil? && col.default.nil?

      changes << "change_column :#{new_table_name}, :#{c}, " +
                 ([":#{spec.sql_type}"] + format_options(change_spec, spec.sql_type, changing: true)).join(", ")
      back = change_column_back(current_table_name, col_name)
      undo_changes << back unless back.blank?
    end
  end.compact

  index_changes, undo_index_changes = change_indexes(model, current_table_name)
  fk_changes, undo_fk_changes = if ActiveRecord::Base.connection.class.name.match?(/SQLite3Adapter/)
                                  [[], []]
                                else
                                  change_foreign_key_constraints(model, current_table_name)
                                end

  [(renames + adds + removes + changes) * "\n",
   (undo_renames + undo_adds + undo_removes + undo_changes) * "\n",
   index_changes * "\n",
   undo_index_changes * "\n",
   fk_changes * "\n",
   undo_fk_changes * "\n"]
end

#column_options_from_reverted_table(table, col_name) ⇒ Object



562
563
564
565
566
567
568
569
570
571
572
573
# File 'lib/generators/declare_schema/migration/migrator.rb', line 562

def column_options_from_reverted_table(table, col_name)
  revert = revert_table(table)
  if (md = revert.match(/\s*t\.column\s+"#{col_name}",\s+(:[a-zA-Z0-9_]+)(?:,\s+(.*?)$)?/m))
    # Ugly migration
    _, type, options = *md
  elsif (md = revert.match(/\s*t\.([a-z_]+)\s+"#{col_name}"(?:,\s+(.*?)$)?/m))
    # Sexy migration
    _, type, options = *md
    type = ":#{type}"
  end
  [type, options]
end

#connection ⇒ Object



142
143
144
# File 'lib/generators/declare_schema/migration/migrator.rb', line 142

def connection
  self.class.connection
end

#create_constraints(model) ⇒ Object



342
343
344
# File 'lib/generators/declare_schema/migration/migrator.rb', line 342

def create_constraints(model)
  model.constraint_specs.map { |fk| fk.to_add_statement(model.table_name) }
end

#create_field(field_spec, field_name_width) ⇒ Object



346
347
348
349
350
# File 'lib/generators/declare_schema/migration/migrator.rb', line 346

def create_field(field_spec, field_name_width)
  options = fk_field_options(field_spec.model, field_spec.name).merge(field_spec.sql_options)
  args = [field_spec.name.inspect] + format_options(options, field_spec.sql_type)
  format("  t.%-*s %s", field_name_width, field_spec.sql_type, args.join(', '))
end

#create_indexes(model) ⇒ Object



338
339
340
# File 'lib/generators/declare_schema/migration/migrator.rb', line 338

def create_indexes(model)
  model.index_specs.map { |i| i.to_add_statement(model.table_name) }
end

#create_table(model) ⇒ Object



316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
331
332
333
334
335
336
# File 'lib/generators/declare_schema/migration/migrator.rb', line 316

def create_table(model)
  longest_field_name = model.field_specs.values.map { |f| f.sql_type.to_s.length }.max
  disable_auto_increment = model.respond_to?(:disable_auto_increment) && model.disable_auto_increment
  primary_key_option =
    if model.primary_key.blank? || disable_auto_increment
      ", id: false"
    elsif model.primary_key == "id"
      ", id: :bigint"
    else
      ", primary_key: :#{model.primary_key}"
    end
  (["create_table :#{model.table_name}#{primary_key_option} do |t|"] +
  [(disable_auto_increment ? "  t.integer :id, limit: 8, auto_increment: false, primary_key: true" : nil)] +
   model.field_specs.values.sort_by(&:position).map { |f| create_field(f, longest_field_name) } +
   ["end"] + (if Migrator.disable_indexing
                []
              else
                create_indexes(model) +
                   create_constraints(model)
              end)).compact * "\n"
end

#drop_index(table, name) ⇒ Object



476
477
478
479
480
# File 'lib/generators/declare_schema/migration/migrator.rb', line 476

def drop_index(table, name)
  # see https://hobo.lighthouseapp.com/projects/8324/tickets/566
  # for why the rescue exists
  "remove_index :#{table}, name: :#{name} rescue ActiveRecord::StatementInvalid"
end

#extract_column_renames!(to_add, to_remove, table_name) ⇒ Object



202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
# File 'lib/generators/declare_schema/migration/migrator.rb', line 202

def extract_column_renames!(to_add, to_remove, table_name)
  if renames
    to_rename = {}
    if (column_renames = renames&.[](table_name.to_sym))
      # A hash of table renames has been provided

      column_renames.each_pair do |old_name, new_name|
        if to_add.delete(new_name.to_s) && to_remove.delete(old_name.to_s)
          to_rename[old_name.to_s] = new_name.to_s
        else
          raise Error, "Invalid rename specified: #{old_name} => #{new_name}"
        end
      end
    end
    to_rename

  elsif @ambiguity_resolver
    @ambiguity_resolver.call(to_add, to_remove, "column", "#{table_name}.")

  else
    raise Error, "Unable to resolve migration ambiguities in table #{table_name}"
  end
end

#extract_table_renames!(to_create, to_drop) ⇒ Object

return a hash of table renames and modifies the passed arrays so that renamed tables are no longer listed as to_create or to_drop



177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
# File 'lib/generators/declare_schema/migration/migrator.rb', line 177

def extract_table_renames!(to_create, to_drop)
  if renames
    # A hash of table renames has been provided

    to_rename = {}
    renames.each_pair do |old_name, new_name|
      new_name = new_name[:table_name] if new_name.is_a?(Hash)
      next unless new_name

      if to_create.delete(new_name.to_s) && to_drop.delete(old_name.to_s)
        to_rename[old_name.to_s] = new_name.to_s
      else
        raise Error, "Invalid table rename specified: #{old_name} => #{new_name}"
      end
    end
    to_rename

  elsif @ambiguity_resolver
    @ambiguity_resolver.call(to_create, to_drop, "table", nil)

  else
    raise Error, "Unable to resolve migration ambiguities"
  end
end

#fk_field_options(model, field_name) ⇒ Object



529
530
531
532
533
534
535
536
537
538
539
540
541
542
543
544
545
546
547
548
# File 'lib/generators/declare_schema/migration/migrator.rb', line 529

def fk_field_options(model, field_name)
  foreign_key = model.constraint_specs.find { |fk| field_name == fk.foreign_key.to_s }
  if foreign_key && (parent_table = foreign_key.parent_table_name)
    parent_columns = connection.columns(parent_table) rescue []
    pk_limit  =
      if (pk_column = parent_columns.find { |column| column.name.to_s == "id" }) # right now foreign keys assume id is the target
        if Rails::VERSION::MAJOR <= 4
          pk_column.cast_type.limit
        else
          pk_column.limit
        end
      else
        8
      end

    { limit: pk_limit }
  else
    {}
  end
end

#format_options(options, type, changing: false) ⇒ Object



512
513
514
515
516
517
518
519
520
521
522
523
524
525
526
527
# File 'lib/generators/declare_schema/migration/migrator.rb', line 512

def format_options(options, type, changing: false)
  options.map do |k, v|
    unless changing
      next if k == :limit && (type == :decimal || v == native_types[type][:limit])
      next if k == :null && v == true
    end

    next if k == :limit && type == :text && !::DeclareSchema::Model::FieldSpec.mysql_text_limits?

    if k.is_a?(Symbol)
      "#{k}: #{v.inspect}"
    else
      "#{k.inspect} => #{v.inspect}"
    end
  end.compact
end

#generate ⇒ Object



246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
# File 'lib/generators/declare_schema/migration/migrator.rb', line 246

def generate
  models, db_tables = models_and_tables
  models_by_table_name = {}
  models.each do |m|
    if !models_by_table_name.has_key?(m.table_name)
      models_by_table_name[m.table_name] = m
    elsif m.superclass == models_by_table_name[m.table_name].superclass.superclass
      # we need to ensure that models_by_table_name contains the
      # base class in an STI hierarchy
      models_by_table_name[m.table_name] = m
    end
  end
  # generate shims for HABTM models
  habtm_tables.each do |name, refls|
    models_by_table_name[name] = HabtmModelShim.from_reflection(refls.first)
  end
  model_table_names = models_by_table_name.keys

  to_create = model_table_names - db_tables
  to_drop = db_tables - model_table_names - self.class.always_ignore_tables
  to_change = model_table_names
  to_rename = extract_table_renames!(to_create, to_drop)

  renames = to_rename.map do |old_name, new_name|
    "rename_table :#{old_name}, :#{new_name}"
  end * "\n"
  undo_renames = to_rename.map do |old_name, new_name|
    "rename_table :#{new_name}, :#{old_name}"
  end * "\n"

  drops = to_drop.map do |t|
    "drop_table :#{t}"
  end * "\n"
  undo_drops = to_drop.map do |t|
    revert_table(t)
  end * "\n\n"

  creates = to_create.map do |t|
    create_table(models_by_table_name[t])
  end * "\n\n"
  undo_creates = to_create.map do |t|
    "drop_table :#{t}"
  end * "\n"

  changes = []
  undo_changes = []
  index_changes = []
  undo_index_changes = []
  fk_changes = []
  undo_fk_changes = []
  to_change.each do |t|
    model = models_by_table_name[t]
    table = to_rename.key(t) || model.table_name
    if table.in?(db_tables)
      change, undo, index_change, undo_index, fk_change, undo_fk = change_table(model, table)
      changes << change
      undo_changes << undo
      index_changes << index_change
      undo_index_changes << undo_index
      fk_changes << fk_change
      undo_fk_changes << undo_fk
    end
  end

  up = [renames, drops, creates, changes, index_changes, fk_changes].flatten.reject(&:blank?) * "\n\n"
  down = [undo_changes, undo_renames, undo_drops, undo_creates, undo_index_changes, undo_fk_changes].flatten.reject(&:blank?) * "\n\n"

  [up, down]
end

#habtm_tables ⇒ Object

list habtm join tables



151
152
153
154
155
156
157
158
159
# File 'lib/generators/declare_schema/migration/migrator.rb', line 151

def habtm_tables
  reflections = Hash.new { |h, k| h[k] = [] }
  ActiveRecord::Base.send(:descendants).map do |c|
    c.reflect_on_all_associations(:has_and_belongs_to_many).each do |a|
      reflections[a.join_table] << a
    end
  end
  reflections
end

#load_rails_models ⇒ Object

TODO: Add an application callback (maybe an initializer in a special group?) that the application can use to load other models that live in the database, to support DeclareSchema migrations for them.



126
127
128
129
130
131
# File 'lib/generators/declare_schema/migration/migrator.rb', line 126

def load_rails_models
  ActiveRecord::Migration.verbose = false

  Rails.application.eager_load!
  Rails::Engine.subclasses.each(&:eager_load!)
end

#models_and_tables ⇒ Object

Returns an array of model classes and an array of table names that generation needs to take into account



163
164
165
166
167
168
169
170
171
172
173
# File 'lib/generators/declare_schema/migration/migrator.rb', line 163

def models_and_tables
  ignore_model_names = Migrator.ignore_models.map { |model| model.to_s.underscore }
  all_models = table_model_classes
  declare_schema_models = all_models.select do |m|
    (m.name['HABTM_'] ||
      (m.include_in_migration if m.respond_to?(:include_in_migration))) && !m.name.underscore.in?(ignore_model_names)
  end
  non_declare_schema_models = all_models - declare_schema_models
  db_tables = connection.tables - Migrator.ignore_tables.map(&:to_s) - non_declare_schema_models.map(&:table_name)
  [declare_schema_models, db_tables]
end

#native_types ⇒ Object



146
147
148
# File 'lib/generators/declare_schema/migration/migrator.rb', line 146

def native_types
  self.class.native_types
end

#remove_foreign_key(old_table_name, fk_name) ⇒ Object



508
509
510
# File 'lib/generators/declare_schema/migration/migrator.rb', line 508

def remove_foreign_key(old_table_name, fk_name)
  "remove_foreign_key('#{old_table_name}', name: '#{fk_name}')"
end

#revert_column(table, column) ⇒ Object



580
581
582
583
# File 'lib/generators/declare_schema/migration/migrator.rb', line 580

def revert_column(table, column)
  type, options = column_options_from_reverted_table(table, column)
  "add_column :#{table}, :#{column}, #{type}#{', ' + options.strip if options}"
end

#revert_table(table) ⇒ Object



550
551
552
553
554
555
556
557
558
559
560
# File 'lib/generators/declare_schema/migration/migrator.rb', line 550

def revert_table(table)
  res = StringIO.new
  schema_dumper_klass = case Rails::VERSION::MAJOR
                        when 4
                          ActiveRecord::SchemaDumper
                        else
                          ActiveRecord::ConnectionAdapters::SchemaDumper
                        end
  schema_dumper_klass.send(:new, ActiveRecord::Base.connection).send(:table, table, res)
  res.string.strip.gsub("\n  ", "\n")
end

#table_model_classes ⇒ Object

Returns an array of model classes that directly extend ActiveRecord::Base, excluding anything in the CGI module



135
136
137
138
139
140
# File 'lib/generators/declare_schema/migration/migrator.rb', line 135

def table_model_classes
  load_rails_models
  ActiveRecord::Base.send(:descendants).select do |klass|
    klass.base_class == klass && !klass.name.starts_with?("CGI::")
  end
end